package sialocacao.sala.database;

/**
 * <p>Title: SalaDatabase.java </p>
 * <p>Description: Implementa a camada de persistncia de sala. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.sql.*;
import java.util.*;

import sialocacao.*;
import sialocacao.sala.*;
import sialocacao.sala.excecoes.*;

public class SalaDatabase extends Database implements ISalaDatabase {

  /*
   * Construtor para carregar driver jdbc.
   */
  public SalaDatabase() throws ClassNotFoundException {
    super();
  }

  /**
   * Construtor para definir driver, url, usurio e senha,
   * e carregar driver jdbc.
   */
  public SalaDatabase(String driver, String url, String user, String password)
      throws ClassNotFoundException, IllegalArgumentException {

    super(driver, url, user, password);
  }

  /*
   * Insere sala no sistema.
   * @param descricao
   * @param localizacao
   * @param observacao
   */
  public void inserirSala(String descricao, String localizacao, String observacao)
    throws SIAlocacaoException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement("INSERT INTO sala (descricao," +
           "localizacao, observacao) VALUES (?, ?, ?)");

      cmd.setString(1, descricao);
      cmd.setString(2, localizacao);
      cmd.setString(3, observacao);

      cmd.executeUpdate();

    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Altera sala no sistema.
   * @param sala
   */
  public void alterarSala(Sala sala) throws SIAlocacaoException,
      SalaInexistenteException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement(
          "UPDATE sala SET descricao=?, localizacao=?, observacao=? " +
          "WHERE codigo=?");

      cmd.setString(1, sala.getDescricao());
      cmd.setString(2, sala.getLocalizacao());
      cmd.setString(3, sala.getObservacao());
      cmd.setInt(4, sala.getCodigo());

      int num = cmd.executeUpdate();

      if (num == 0) {
        throw new SalaInexistenteException(sala.getCodigo());
      }

    } catch (SalaInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Exclui sala do sistema pelo cdigo.
   * @param codigo
   */
  public void excluirSala(int codigo) throws SIAlocacaoException,
      SalaInexistenteException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement("DELETE FROM sala WHERE " +
                                           "codigo=?");

      cmd.setInt(1, codigo);

      int num = cmd.executeUpdate();

      if (num == 0) {
        throw new SalaInexistenteException(codigo);
      }

    } catch (SalaInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Busca sala no sistema pelo cdigo.
   * @param codigo
   * @return Sala
   */
  public Sala buscarSala(int codigo) throws SIAlocacaoException,
      SalaInexistenteException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement("SELECT descricao, localizacao, "+
           "observacao FROM sala WHERE codigo=?");

      cmd.setInt(1, codigo);

      ResultSet result = cmd.executeQuery();

      Sala Sala;

      if (result.next()) {
        Sala = new Sala(codigo,
                  result.getString("descricao"),
                  result.getString("localizacao"),
                  result.getString("observacao"));
      } else {
        throw new SalaInexistenteException(codigo);
      }

      return Sala;

    } catch (SalaInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Busca sala no sistema pelo cdigo.
   * @param descricao
   * @return Sala
   */
  public Sala buscarSala(String descricao) throws SIAlocacaoException,
      SalaInexistenteException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement("SELECT codigo, localizacao, "+
           "observacao FROM sala WHERE descricao=?");

      cmd.setString(1, descricao);

      ResultSet result = cmd.executeQuery();

      Sala Sala;

      if (result.next()) {
        Sala = new Sala(result.getInt("codigo"),
                  descricao,
                  result.getString("localizacao"),
                  result.getString("observacao"));
      } else {
        throw new SalaInexistenteException(descricao);
      }

      return Sala;

    } catch (SalaInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Lista salas do sistema.
   * @return ArrayList
   */
  public ArrayList listarSalas() throws SIAlocacaoException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement("SELECT * FROM sala");

      ResultSet result = cmd.executeQuery();

      ArrayList salas =
          new ArrayList();

      while (result.next()) {
        salas.add(
                  new Sala(result.getInt("codigo"),
                     result.getString("descricao"),
                     result.getString("localizacao"),
                     result.getString("observacao")));
      }

      return salas;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }
}