package sialocacao.disciplina.database;

/**
 * <p>Title: DisciplinaDatabase.java </p>
 * <p>Description: Implementa a camada de persistncia de Disciplina. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.sql.*;
import java.util.*;

import sialocacao.*;
import sialocacao.disciplina.*;
import sialocacao.disciplina.excecoes.*;

public class DisciplinaDatabase extends Database implements IDisciplinaDatabase {

  /*
   * Construtor para carregar driver jdbc.
   */
  public DisciplinaDatabase() throws ClassNotFoundException {
    super();
  }

  /**
   * Construtor para definir driver, url, usurio e senha,
   * e carregar driver jdbc.
   */
  public DisciplinaDatabase(String driver, String url, String user, String password)
      throws ClassNotFoundException, IllegalArgumentException {

    super(driver, url, user, password);
  }

  /*
   * Insere disciplina no sistema.
   * @param descricao
   * @param cargaHoraria
   * @param ementa
   * @param area
   */
  public void inserirDisciplina(String descricao, int cargaHoraria, String ementa, String area)
      throws SIAlocacaoException {

    try {
      openConnection();
      PreparedStatement cmd = getConnection().prepareStatement("INSERT INTO "+
      "disciplina (descricao, cargaHoraria, ementa, area) VALUES (?, ?, ?, ?)");

      cmd.setString(1, descricao);
      cmd.setInt(2, cargaHoraria);
      cmd.setString(3, ementa);
      cmd.setString(4, area);

      cmd.executeUpdate();

    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {System.out.println("\nnao fechei\n");
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Altera disciplina no sistema.
   * @param disciplina
   */
  public void alterarDisciplina(Disciplina disciplina)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    try {
      openConnection();
      PreparedStatement cmd = getConnection().prepareStatement( "UPDATE disciplina SET descricao=?, cargaHoraria=?, ementa=?, area=? WHERE codigo=?");

      cmd.setString(1, disciplina.getDescricao());
      cmd.setInt(2, disciplina.getCargaHoraria());
      cmd.setString(3, disciplina.getEmenta());
      cmd.setString(4, disciplina.getArea());
      cmd.setInt(5, disciplina.getCodigo());

      int num = cmd.executeUpdate();

      if (num == 0) {
        throw new DisciplinaInexistenteException(disciplina.getCodigo());
      }

    } catch (DisciplinaInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {System.out.println("\nnao fechei\n");
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Exclui disciplina do sistema pelo cdigo.
   * @param codigo
   */
  public void excluirDisciplina(int codigo)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    try {
      openConnection();
      PreparedStatement cmd = getConnection().prepareStatement("DELETE FROM disciplina WHERE codigo=?");

      cmd.setInt(1, codigo);

      int num = cmd.executeUpdate();
      if (num == 0) {
        throw new DisciplinaInexistenteException(codigo);
      }

    }catch (DisciplinaInexistenteException e) {
      throw e;
    }catch (SQLException e) {
      if(e.getMessage().indexOf("referential integrity") >= 0)
        throw new DisciplinaAlocadaException();
    }
    catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {System.out.println("\nnao fechei\n");
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Busca disciplina no sistema pelo cdigo.
   * @param codigo
   * @return Disciplina
   */
  public Disciplina buscarDisciplina(int codigo)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    try {
      openConnection();
      PreparedStatement cmd = getConnection().prepareStatement("SELECT descricao, cargaHoraria, ementa, area FROM disciplina WHERE codigo=?");

      cmd.setInt(1, codigo);

      ResultSet result = cmd.executeQuery();
      Disciplina disciplina;

      if (result.next()) {
        disciplina = new Disciplina (codigo,
                        result.getString("descricao"),
                        result.getInt("cargaHoraria"),
                        result.getString("ementa"),
                        result.getString("area"));
      } else {
        throw new DisciplinaInexistenteException(codigo);
      }

      return disciplina;
    } catch (DisciplinaInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {System.out.println("\nnao fechei\n");
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Busca disciplina no sistema pelo cdigo.
   * @param descricao
   * @return Disciplina
   */
  public Disciplina buscarDisciplina(String descricao)
      throws SIAlocacaoException, DisciplinaInexistenteException {

    try {
      openConnection();
      PreparedStatement cmd = getConnection().prepareStatement("SELECT codigo,"+
        "cargaHoraria, ementa, area FROM disciplina WHERE descricao=?");

      cmd.setString(1, descricao);

      ResultSet result = cmd.executeQuery();
      Disciplina disciplina;

      if (result.next()) {
        disciplina = new Disciplina (result.getInt("codigo"),
                        descricao,
                        result.getInt("cargaHoraria"),
                        result.getString("ementa"),
                        result.getString("area"));
      } else {
        throw new DisciplinaInexistenteException(descricao);
      }

      return disciplina;
    } catch (DisciplinaInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {System.out.println("\nnao fechei\n");
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
}

  /*
   * Lista disciplinas do sistema.
   * @return ArrayList
   */
  public ArrayList listarDisciplinas() throws
       SIAlocacaoException, DisciplinaInexistenteException {

     try {
       openConnection();
       PreparedStatement cmd = getConnection().prepareStatement("SELECT * FROM disciplina");
       ResultSet result = cmd.executeQuery();
       ArrayList disciplinas = new ArrayList();

       while (result.next()) {
         disciplinas.add(new Disciplina ( result.getInt("codigo"),
             result.getString("descricao"),
             result.getInt("cargaHoraria"),
             result.getString("ementa"),
             result.getString("area")));
       }

       return disciplinas;
     } catch (Exception e) {
       throw new SIAlocacaoException(e.getMessage(), e);
     } finally {
       try {
         if (getConnection() != null) {
           commit();
           closeConnection();
         }
       } catch (Exception e) {System.out.println("\nnao fechei\n");
         throw new SIAlocacaoException(e.getMessage(), e);
       }
     }
   }
 }