package sialocacao.horario.database;

/**
 * <p>Title: HorarioDatabase.java </p>
 * <p>Description: Implementa a camada de persistncia de horario. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.sql.*;
import java.util.*;

import sialocacao.*;
import sialocacao.horario.*;
import sialocacao.horario.excecoes.*;

public class HorarioDatabase extends Database implements IHorarioDatabase {

  /*
   * Construtor para carregar driver jdbc.
   */
  public HorarioDatabase() throws ClassNotFoundException {
    super();
  }

  /**
   * Construtor para definir driver, url, usurio e senha,
   * e carregar driver jdbc.
   */
  public HorarioDatabase(String driver, String url, String user, String password)
      throws ClassNotFoundException, IllegalArgumentException {

    super(driver, url, user, password);
  }

  /*
   * Insere horrio no sistema.
   * @param diaDaSemana
   * @param horaInicio
   * @param horaFim
   */
  public void inserirHorario(String diaDaSemana, String horaInicio, String horaFim)
      throws SIAlocacaoException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement("INSERT INTO horario (diaDaSemana," +
                                           "horaInicio, horaFim) VALUES (?, ?, ?)");

      cmd.setString(1, diaDaSemana);
      cmd.setString(2, horaInicio);
      cmd.setString(3, horaFim);

      cmd.executeUpdate();

    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Altera horrio no sistema.
   * @param horario
   */
  public void alterarHorario(Horario horario) throws SIAlocacaoException,
      HorarioInexistenteException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement(
          "UPDATE horario SET diaDaSemana=?, horaInicio=?, horaFim=? " +
          "WHERE codigo=?");

      cmd.setString(1, horario.getDiaDaSemana());
      cmd.setString(2, horario.getHoraInicio());
      cmd.setString(3, horario.getHoraFim());
      cmd.setInt(4, horario.getCodigo());

      int num = cmd.executeUpdate();

      if (num == 0) {
        throw new HorarioInexistenteException(horario.getCodigo());
      }

    } catch (HorarioInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Exclui horrio do sistema pelo cdigo.
   * @param codigo
   */
  public void excluirHorario(int codigo) throws SIAlocacaoException,
      HorarioInexistenteException, HorarioAlocadoException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement("DELETE FROM horario WHERE " +
                                           "codigo=?");

      cmd.setInt(1, codigo);

      int num = cmd.executeUpdate();

      if (num == 0) {
        throw new HorarioInexistenteException(codigo);
      }

    }catch (SQLException e) {
      if(e.getMessage().indexOf("referential integrity") >= 0)
        throw new HorarioAlocadoException();
    }
    catch (HorarioInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Busca horrio no sistema pelo cdigo.
   * @param codigo
   * @return Horario
   */
  public Horario buscarHorario(int codigo) throws SIAlocacaoException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement("SELECT diaDaSemana, horaInicio, "+
                                           "horaFim FROM horario WHERE codigo=?");

      cmd.setInt(1, codigo);

      ResultSet result = cmd.executeQuery();

      Horario horario;

      if (result.next()) {
        horario =
            new Horario(codigo,
                        result.getString("diaDaSemana"),
                        result.getString("horaInicio"),
                        result.getString("horaFim"));
      } else {
        return null;
      }

      return horario;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Busca horrio no sistema pelo cdigo.
   * @param diaDaSemana
   * @param horaInicio
   * @param horaFim
   * @return Horario
   */
  public Horario buscarHorario(String diaDaSemana, String horaInicio,
    String horaFim) throws SIAlocacaoException {

    try {
      openConnection();

      PreparedStatement cmd =
          getConnection().prepareStatement("SELECT codigo FROM horario WHERE "+
            "diaDaSemana=? AND horaInicio=? AND horaFim=?");

      cmd.setString(1, diaDaSemana);
      cmd.setString(2, horaInicio);
      cmd.setString(3, horaFim);

      ResultSet result = cmd.executeQuery();

      Horario horario;

      if (result.next()) {
        horario =
            new Horario(result.getInt("codigo"),
                        diaDaSemana,
                        horaInicio,
                        horaFim);
      } else {
        return null;
      }

      return horario;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {System.out.println("\nnao fechei\n");
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Lista horaiors do sistema.
   * @return ArrayList
   */
  public ArrayList listarHorarios() throws SIAlocacaoException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement("SELECT * FROM horario ORDER BY "+
            "horaInicio");

      ResultSet result = cmd.executeQuery();

      ArrayList horarios =
          new ArrayList();

      while (result.next()) {
        horarios.add(
            new Horario(result.getInt("codigo"),
                        result.getString("diaDaSemana"),
                        result.getString("horaInicio"),
                        result.getString("horaFim")));
      }

      return horarios;

    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {System.out.println("\nnao fechei\n");
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Lista horaiors do sistema.
   * @return ArrayList
   */
  public ArrayList listarHorariosDistintos() throws SIAlocacaoException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement("SELECT DISTINCT ON horaInicio * "+
            "FROM horario ORDER BY horaInicio");

      ResultSet result = cmd.executeQuery();

      ArrayList horarios =
          new ArrayList();

      while (result.next()) {
        horarios.add(
            new Horario(result.getInt("codigo"),
                        result.getString("diaDaSemana"),
                        result.getString("horaInicio"),
                        result.getString("horaFim")));
      }

      return horarios;

    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {System.out.println("\nnao fechei\n");
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }
}