package sialocacao.professor.database;

/**
 * <p>Title: HorarioDatabase.java </p>
 * <p>Description: Implementa a camada de persistncia de professor. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Fbrica III </p>
 * @author: Carlos A. Rego Andrade
 *          Fbio de Carvalho Dias Ferreira
 *          Luiz Eduardo Cunha Leite
 *          Taciana Amorim Vanderlei
 *          Vivianne da Nbrega Medeiros
 * @version 1.0
 */

import java.sql.*;
import java.util.*;

import sialocacao.*;
import sialocacao.professor.*;
import sialocacao.horario.*;
import sialocacao.professor.excecoes.*;

public class ProfessorDatabase extends Database implements IProfessorDatabase {

  /*
   * Construtor para carregar driver jdbc.
   */
  public ProfessorDatabase() throws ClassNotFoundException {
    super();
  }

  /**
   * Construtor para definir driver, url, usurio e senha,
   * e carregar driver jdbc.
   */
  public ProfessorDatabase(String driver, String url, String user, String password)
      throws ClassNotFoundException, IllegalArgumentException {

    super(driver, url, user, password);
  }

  /*
   * Insere professor no sistema.
   * @param nome
   * @param area
   */
  public void inserirProfessor(String nome, String area) throws SIAlocacaoException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement("INSERT INTO professor (nome," +
                                           "area) VALUES (?, ?)");

      cmd.setString(1, nome);
      cmd.setString(2, area);

      cmd.executeUpdate();

    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {System.out.println("\nnao fechei\n");
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Altera professor no sistema.
   * @param professor
   */
  public void alterarProfessor(Professor professor) throws SIAlocacaoException,
      ProfessorInexistenteException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement(
          "UPDATE professor SET nome=?, area=? " +
          "WHERE codigo=?");

      cmd.setString(1, professor.getNome());
      cmd.setString(2, professor.getArea());
      cmd.setInt(3, professor.getCodigo());

      int num = cmd.executeUpdate();

      if (num == 0) {
        throw new ProfessorInexistenteException(professor.getCodigo());
      }

    } catch (ProfessorInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {System.out.println("\nnao fechei\n");
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Exclui professor do sistema pelo cdigo.
   * @param codigo
   */
  public void excluirProfessor(int codigo) throws SIAlocacaoException,
      ProfessorInexistenteException, ProfessorAlocadoException{

    try {
      openConnection();


      PreparedStatement cmd =
          getConnection().prepareStatement("DELETE FROM professor WHERE " +
                                           "codigo=?");
      cmd.setInt(1, codigo);

      int num = cmd.executeUpdate();

      if (num == 0) {
        throw new ProfessorInexistenteException(codigo);
      }

    }catch (ProfessorInexistenteException e) {
      throw e;
    }catch (SQLException e) {
      if(e.getMessage().indexOf("referential integrity") >= 0)
        throw new ProfessorAlocadoException();
    }
    catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {System.out.println("\nnao fechei\n");
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

  /*
   * Busca professor no sistema pelo cdigo.
   * @param codigo
   * @return Professor
   */
  public Professor buscarProfessor(int codigo) throws SIAlocacaoException,
      ProfessorInexistenteException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement("SELECT nome, area FROM professor " +
                                           "WHERE codigo=?");

      cmd.setInt(1, codigo);

      ResultSet result = cmd.executeQuery();

      Professor professor;

      if (result.next()) {
        professor =
            new Professor(codigo,
                          result.getString("nome"),
                          result.getString("area"));
      } else {
        throw new ProfessorInexistenteException(codigo);
      }

      return professor;
    } catch (ProfessorInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {System.out.println("\nnao fechei\n");
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }

/*
   * Busca professor no sistema pelo nome.
   * @param nome
   * @return Professor
   */
  public Professor buscarProfessor(String nome) throws SIAlocacaoException,
      ProfessorInexistenteException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement("SELECT codigo, area FROM professor " +
                                           "WHERE nome=?");

      cmd.setString(1, nome);

      ResultSet result = cmd.executeQuery();

      Professor professor;

      if (result.next()) {
        professor =
            new Professor(result.getInt("codigo"),
                          nome,
                          result.getString("area"));
      } else {
        throw new ProfessorInexistenteException(nome);
      }

      return professor;
    } catch (ProfessorInexistenteException e) {
      throw e;
    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {System.out.println("\nnao fechei\n");
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
}

  /*
   * Lista professores do sistema.
   * @return ArrayList
   */
  public ArrayList listarProfessores() throws SIAlocacaoException {

    try {
      openConnection();
      PreparedStatement cmd =
          getConnection().prepareStatement("SELECT * FROM professor");

      ResultSet result = cmd.executeQuery();

      ArrayList professores =
          new ArrayList();

      while (result.next()) {
        professores.add(
            new Professor(result.getInt("codigo"),
                          result.getString("nome"),
                          result.getString("area")));
      }

      return professores;

    } catch (Exception e) {
      throw new SIAlocacaoException(e.getMessage(), e);
    } finally {
      try {
        if (getConnection() != null) {
          commit();
          closeConnection();
        }
      } catch (Exception e) {System.out.println("\nnao fechei\n");
        throw new SIAlocacaoException(e.getMessage(), e);
      }
    }
  }
}
